@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##############################################################
 # smscon - remote control library for Nokia N900 phone       #
-# $Id: smsconlib.py 160 2012-01-22 22:15:30Z yablacky $
+# $Id: smsconlib.py 165 2012-04-14 07:10:08Z yablacky $
 ##############################################################
 
 # standard modules
@@ -34,7 +34,7 @@ NAME = 'smscon'
 VERSION = '999.1'   # for trunk and private builds etc.
 
 NAME, VERSION = getSysBuildNameAndVersion(NAME, VERSION,
-    headerInfo  = '$Header: https://vcs.maemo.org/svn/smscon/branches/smscon_0.9.16/src/opt/smscon/smsconlib.py 160 2012-01-22 22:15:30Z yablacky $',
+    headerInfo  = '$Header: https://vcs.maemo.org/svn/smscon/branches/smscon_0.9.17/src/opt/smscon/smsconlib.py 165 2012-04-14 07:10:08Z yablacky $',
     dirLevels   = 3 )
     
 XDEBUG   = False        # cross debug (use only if not running on phone)
@@ -213,77 +213,74 @@ def SendSMSviaGSMmodem(RecipientNumber, Message, TryCountMax = 2):
     assert(len(Message) <= 160)
 
     RecipientNumber = NormalizePhoneNumber(RecipientNumber)
-    DelaySeconds = 2
 
     modem = None
     TryCount = 0
     while TryCount < TryCountMax:
         TryCount += 1
+        LOGGER.debug('GSM modem: try count=%d/%d' % (TryCount, TryCountMax))
 
         if modem:
             modem.sendeof()
+            time.sleep(1)
             modem.kill(9)
-            time.sleep(DelaySeconds)
-        modem = pexpect.spawn('pnatd', [], timeout = 2) # start pnatd
+            time.sleep(1)
+        modem = pexpect.spawn('pnatd', [], timeout = 8) # start pnatd
         #modem.logfile = sys.stdout
+        time.sleep(1)   # must give pnatd a chance to start reading from stdin.
 
         try:
-            time.sleep(1)
-            modem.send('AT\r') # GSM modem returns 'OK' if working
-            modem.readline()
-            response = modem.readline().strip('\r\n')
+            cmd = 'AT'
+            LOGGER.debug('GSM modem: %s' % cmd)
+            modem.send(cmd + '\r') and modem.readline() # eat echo
+            response = modem.readline().strip()
+            if response != 'OK':
+                LOGGER.debug('GSM modem: expected "OK" - got "%s".' % response)
+                continue
+
+            cmd = 'AT+CMGF=1'
+            LOGGER.debug('GSM modem: %s' % cmd)
+            modem.send(cmd + '\r') and modem.readline() # eat echo
+            response = modem.readline().strip()
+            if response != 'OK':
+                LOGGER.debug('GSM modem: expected "OK" - got "%s".' % response)
+                continue
+
+            cmd = 'AT+CMGS="%s"' % RecipientNumber
+            LOGGER.debug('GSM modem: %s' % cmd)
+            modem.send(cmd + '\r') and modem.readline() # eat echo
+            modem.expect_exact('> ')
+
+            cmd = '(sms text)'
+            LOGGER.debug('GSM modem: %s' % cmd)
+            modem.send( Message + chr(26) ) and modem.readline()    # message plus ctrl-Z; eat echo
+
+            response = modem.readline().strip()
+            if not response.startswith('+CMGS:'):
+                LOGGER.debug('GSM modem: expected "+CMGS:" - got "%s".' % response)
+                continue 
+
+            cmd = '(empty line)'
+            modem.readline()        # eat empty line
+            response = modem.readline().strip()
+            if response != 'OK':
+                LOGGER.debug('GSM modem: expected "OK" - got "%s".' % response)
+                continue 
+
+        except pexpect.EOF:
+            LOGGER.debug('GSM modem: got eof after %s' % cmd)
+            continue
         except pexpect.TIMEOUT:
-            LOGGER.debug('GSM modem "AT" timeout.')
+            LOGGER.debug('GSM modem: timeout after %s' % cmd)
             continue
-
-        if response != 'OK':
-            LOGGER.debug('GSM modem did not reply correctly (%s).' % response)
-            continue
-
-        try:
-            time.sleep(1)
-            modem.send('AT+CMGF=1\r') # GSM modem returns 'OK' if working
-            modem.readline()
-            response = modem.readline().strip('\r\n')
-        except pexpect.TIMEOUT:
-            LOGGER.debug('GSM modem "AT+CMGF" timeout.')
-            continue
-
-        if response != 'OK':
-            LOGGER.debug('GSM modem did not reply correctly (%s).' % response)
-            continue 
-
-        try:
-            time.sleep(1)
-            modem.send('AT+CMGS="%s"\r' % RecipientNumber) # GSM modem returns '> ' if working
-            modem.readline()
-        except pexpect.TIMEOUT:
-            LOGGER.debug('GSM modem "AT+CMGS" timeout.')
-            continue
-
-        time.sleep(1)
-        modem.send( '%s' % Message + chr(26) ) # set SMS message and Ctrl + Z to send message
-
-        try:
-            time.sleep(1)
-            modem.readline()
-            response = modem.readline().strip('\r\n')
-        except pexpect.TIMEOUT:
-            LOGGER.debug('GSM modem response "+CMGS:" timeout.')
-            continue
-
-        if not response.startswith('+CMGS:'):
-            LOGGER.debug('GSM modem response "+CMGS:" failed (%s).' % response)
-            continue 
-
-        modem.readline()
-        response = modem.readline().strip('\r\n')
-        if response != 'OK':
-            LOGGER.debug('GSM modem response "OK" failed (%s).' % response)
-            continue 
-
+        LOGGER.debug('GSM modem: success.')
         return True
 
+    LOGGER.debug('GSM modem: giving up.')
+    if modem:
+        modem.sendeof()
+        time.sleep(1)
+        modem.kill(9)
     return False
 
 ##############################################################
